//! Components for queries.

/// Grouping component
pub(crate) mod grouping;

/// Facet count component
pub(crate) mod facet_set;

/// Json facet component
pub(crate) mod json_facet;
/// Stats component
pub(crate) mod stats;
